/*
 Copyright (c) 2006 - 2007 Eric J. Feminella  <eric@ericfeminella.com>
 All rights reserved.
  
 Permission is hereby granted, free of charge, to any person obtaining a copy 
 of this software and associated documentation files (the "Software"), to deal 
 in the Software without restriction, including without limitation the rights 
 to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the Software is furnished 
 to do so, subject to the following conditions:
 
 The above copyright notice and this permission notice shall be included in all 
 copies or substantial portions of the Software.
 
 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
 INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
 PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION 
 OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
 SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

 @internal
 */
 
package com.ericfeminella.utils.iterators 
{
    /**
     * Concrete Iterator implementation which provides methods 
     * for iterating over an array
     */
	public class ArrayIterator implements IIterator
	{
		/**
		 * The aggregate [Array] in which to traverse
		 */		
		private var items:Array;
		
		/**
		 * The index of the current item in the Array
		 */	
		private var index:int = 0;
		
		/**
		 * Instantiates a new ArrayIterator instance
	     * 
         * @example A basic ArrayIterator example is as follows:
         * 
         * <listing version="3.0" >
         * 
	     * var array:Array = new Array("A","B","C");
	     * var it:IIterator = new ArrayIterator( array );
	     * 
	     * while ( it.hasNext() )
	     * {
	     *     trace( it.next(), it.position());
	     * }
	     * 
	     * //outputs:
	     * //A, 0
	     * //B, 1
	     * //C, 2
	     * </listing>
	     * 
		 * @param the Array in which to iterate over
		 */
		public function ArrayIterator(array:Array = null)
		{
			this.items = array;
		}
		
		/**
		 * Sets the array in which the ArrayIterator instance is 
		 * to iterator over
		 * 
		 * @inheritDoc
		 */
		public function setAggregate(aggregate:Object = null) : void
		{
        	if (aggregate != null)
        	{
	        	this.items = aggregate as Array;     		
        	}			
		}
		
		/**
		 * Determines if there are elements remaining in the aggregate
		 * 
		 * @inheritDoc
		 */	
		public function hasNext() : Boolean
		{
			return this.index < this.items.length;
		}
		
		/**
		 * Returns the next item element in the array
		 * 
		 * @return  Next array element based on the current index
		 */	
		public function next() : *
		{
			return this.items[ this.index++ ];
		}
		
		/**
		 * Resets the index of the ArrayIterator instance to 0
		 */ 
	    public function reset() : void
	    {
	    	this.index = 0;
	    }
	    
		/**
		 * Determines the indexed position of the aggreagate
		 * 
		 * @inheritDoc
		 */
	    public function position() : int
		{
			return this.index;
		}	
	}
}


