/*
 Copyright (c) 2006 - 2007 Eric J. Feminella  <eric@ericfeminella.com>
 All rights reserved.
  
 Permission is hereby granted, free of charge, to any person obtaining a copy 
 of this software and associated documentation files (the "Software"), to deal 
 in the Software without restriction, including without limitation the rights 
 to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the Software is furnished 
 to do so, subject to the following conditions:
 
 The above copyright notice and this permission notice shall be included in all 
 copies or substantial portions of the Software.
 
 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
 INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
 PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION 
 OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
 SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

 @internal
 */
 
package com.ericfeminella.utils.iterators 
{
    /**
     * Concrete Iterator implementation which provides methods for 
     * iterating over a generic object
     */
	public class ObjectIterator implements IIterator 
	{	
		/**
		 * The aggregate object in which to iterate over
		 */	
		private var source:Array;
		
		/**
		 * The index of the current item in the iteration
		 */	
		private var index:int = 0;
		
		/**
		 * Instantiates a new ObjectIterator instance
		 * 
		 * @param the aggregate object in which to iterate over
		 */
		public function ObjectIterator(object:Object = null)
		{
			this.setAggregate( object );		
		}
		
		/**
		 * Sets the aggregate object in which to iterate over
		 * 
		 * @param the object instance to traverse
		 */
		public function setAggregate(aggregate:Object = null) : void
		{
        	if (aggregate != null)
        	{
	            this.source = new Array();
	            
				for (var prop:* in aggregate){
					this.source.push( aggregate[prop] );
				} 		
        	}
		}
		
		/**
		 * Determines if there are properties remaining in the Object
		 * 
		 * @return  true if a index remains in the Object, false if not
		 */	
		public function hasNext() : Boolean
		{
			return this.index < this.source.length;
		}
		
		/**
		 * Returns the next index value in the Object
		 * 
		 * @inheritDoc
		 */	
		public function next() : *
		{
			return this.source[this.index++];
		}
		
		/**
		 * Resets the index of the ObjectIterator to 0
		 */ 
	    public function reset() : void
	    {
	    	this.index = 0;
	    }
	    
		/**
		 * Determines the position of the current element in the aggreagate
		 * 
		 * @inheritDoc
		 */
	    public function position() : int
		{
			return index;
		}
	}
}


